package pt.moth.dependencies.secondPhase;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import pt.moth.dependencies.VarDepCollector;
import pt.moth.dependencies.VarVersion;
import pt.moth.dependencies.firsPhase.GraphLattice;
import pt.moth.util.ImmutableManager;
import pt.moth.util.graphs.DepEdge;
import pt.moth.util.variables.GlobalVar;
import pt.moth.util.variables.IfConditionVar;
import pt.moth.util.variables.IfGlobalVar;
import pt.moth.util.variables.IfLocalVar;
import pt.moth.util.variables.LocalVar;
import pt.moth.util.variables.ReturnRef;
import pt.moth.util.variables.Variable;



import soot.SootClass;
import soot.SootMethod;
import soot.Unit;
import soot.Value;
import soot.jimple.Constant;
import soot.jimple.FieldRef;
import soot.jimple.InvokeExpr;
import soot.jimple.MonitorStmt;
import soot.jimple.Stmt;
import soot.jimple.internal.ImmediateBox;
import soot.jimple.internal.JAssignStmt;
import soot.jimple.internal.JGotoStmt;
import soot.jimple.internal.JIdentityStmt;
import soot.jimple.internal.JIfStmt;
import soot.jimple.internal.JLookupSwitchStmt;
import soot.jimple.internal.JReturnStmt;
import soot.jimple.internal.JReturnVoidStmt;
import soot.jimple.internal.JTableSwitchStmt;
import soot.jimple.internal.JThrowStmt;
import soot.jimple.internal.JimpleLocal;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.graph.UnitGraph;
import soot.toolkits.scalar.ForwardFlowAnalysis;

public class IntraAnalysis extends ForwardFlowAnalysis<Unit, StackLattice>{

	protected InterAnalysis interAnalysis;
	protected SootMethod thisMethod;
	protected StackLattice entryLattice;
	protected Set<ReturnRef> refs;
	public IntraAnalysis(DirectedGraph<Unit> graph, StackLattice entryLat, InterAnalysis interAnalysis) {
		super(graph);
		this.interAnalysis = interAnalysis;
		this.thisMethod = ((UnitGraph) graph).getBody().getMethod();
		this.entryLattice = entryLat.clone();
		
		this.refs = entryLat.getReturnRefs();
		
		doAnalysis();
	}
	
	@Override
	protected void flowThrough(StackLattice inValue, Unit unit, StackLattice outValue) {
		inValue.copy(outValue);
		Stmt stmt = (Stmt) unit;		
		if (stmt.containsInvokeExpr()) {
			SetIfVars setIfVars = new SetIfVars();
			for(SetIfVars set : inValue.toList()){
				for(IfConditionVar var : set){
					setIfVars.add(var);	
				}
			}
			InvokeExpr expr = ((Stmt)unit).getInvokeExpr();
			SootMethod method = expr.getMethod();
			SootClass invokeClass = method.getDeclaringClass(); 
			
			if(interAnalysis.shouldAnalyzeMethod(method) && 
				!ImmutableManager.isImmutable(invokeClass)/* &&
					!method.getDeclaringClass().toString().startsWith("java.") &&
					!method.getDeclaringClass().toString().startsWith("javax.")*/){
				SetIfVars aux = setIfVars.copy();
				for(IfConditionVar var : inValue.oldVars()){
					aux.add(var.copy());
				}
				ReturnRef ref = new ReturnRef(thisMethod, stmt);
				VarVersion dest = new VarVersion(ref);
				VarVersion origin;
				for(IfConditionVar i : aux){
					origin = new VarVersion(i);
					VarDepCollector.addEdge(new DepEdge(origin,dest,true));
				}
				
				Set<ReturnRef> refs = new HashSet<ReturnRef>();
				refs.add(ref);
				interAnalysis.handleMethodInvocation(thisMethod, new StackLattice(setIfVars,refs), stmt);
			}
		}
		// Adicionar novas ifVars ao Lattice
		else if(unit instanceof JIfStmt){
			JIfStmt ifStmt = (JIfStmt) unit;
			List<?> usedVars = ifStmt.getUseBoxes();
			SetIfVars set = new SetIfVars();
			for(Object obj : usedVars){
				if(obj instanceof ImmediateBox){
					Value value = ((ImmediateBox)obj).getValue();
					if(value instanceof JimpleLocal){
						IfConditionVar if_var = new IfLocalVar(thisMethod, (JimpleLocal)value,ifStmt);
						//por a variavel if
						set.add(if_var);
					}
					else if(value instanceof FieldRef){
						GlobalVar glo_var = new GlobalVar((FieldRef)value,thisMethod,true);
						IfConditionVar if_var = new IfGlobalVar(thisMethod, glo_var,ifStmt);
						set.add(if_var);
					}
					else if(!(value instanceof Constant)){
						System.err.println("Dep2.IntraAnalysis: Value is not FieldRef " +
								"or Local : "+value + " "+value.getClass());
						break;
					}	
				}
			}
			outValue.add(set);
		}
		//Adicionar dependencia "ifVar --> Return"
		else if(unit instanceof JReturnStmt){
//			ReturnVar returnVariable = new ReturnVar(thisMethod);
			if(refs != null){
				Set<IfConditionVar> ifVars = extractIfVars(inValue);
				VarVersion origin;
				VarVersion dest;
				for(IfConditionVar var: ifVars){
					origin = new VarVersion(var);
					for(ReturnRef r : refs){
						dest = new VarVersion(r);
						VarDepCollector.addEdge(new DepEdge(origin,dest,true));
//						System.err.println("\nADD EDGE ("+var+","+returnVariable+")" + thisMethod.getName()+ "\n"+ unit);
					}
				}
			}
			else{
				System.err.println("Dep2.IntraAnalysis: Shouldn't Happen, ReturnRef is null\n" +
						"method = "+thisMethod.getSignature()+"\nStmt = "+stmt);
			}
		}
		else if (unit instanceof JAssignStmt) {
			if(inValue.size() != 0){
				Value left = ((JAssignStmt)unit).getLeftOp();
				
				Variable left_var = null; 
				if(left instanceof FieldRef){
					left_var = new GlobalVar((FieldRef)left, thisMethod,true);
				}
				else if(left instanceof JimpleLocal){
					left_var = new LocalVar(thisMethod, (JimpleLocal)left);
				}
				if(left_var != null){
					Set<IfConditionVar> ifVars = extractIfVars(inValue);
					VarVersion origin;
					VarVersion dest = new VarVersion(left_var,getStmtHashCode(stmt));
					for(IfConditionVar var: ifVars){
						origin = new VarVersion(var);
						VarDepCollector.addEdge(new DepEdge(origin,dest,true));
					}
				}
			}
		}
		//We are backing off a if construction (removeLast SetIfVar)
		else if(unit instanceof JGotoStmt){
			outValue.removeLast();
		}
		else if(unit instanceof JReturnVoidStmt){}
		else if(unit instanceof JIdentityStmt){}
		else if(unit instanceof JThrowStmt){}//Throw exception with localVariable (e.g. throw $r0)
		else if(unit instanceof JTableSwitchStmt){}//JTableSwitchStmt used
		else if(unit instanceof MonitorStmt){}//Monitor Statement
		else if(unit instanceof JLookupSwitchStmt){}//JLookupSwitchStmt used
		// Others..
		else {
			System.err.println("Dep2 Intra : ("+thisMethod.getName()+")Other class : (in "+unit.getClass()+") "+unit);
		}
//		System.out.println( unit + "("+unit.getClass()+") - {"+inValue+"}{"+outValue+"}");
	}
	
	/**
	 * Now, it just calls the hashCode method of the soot object, but in the
	 * future it can be more complex function.
	 */
	private int getStmtHashCode(Stmt stmt) {
		return stmt == null? GraphLattice.SPECIAL_HASH : stmt.hashCode();
	}
	
	/**
	 * With the Set of "SetIfVars", returns a set with all the "IfConditionVar" of those Sets.
	 * In other words, transformes a list of lists in a single complete list
	 */
	private Set<IfConditionVar> extractIfVars(StackLattice inValue) {
		Set<IfConditionVar> result = new HashSet<IfConditionVar>();
		for(SetIfVars set : inValue.toList()){
			for(IfConditionVar if_var : set){
				result.add(if_var);
			}
		}
		for(IfConditionVar if_var : inValue.oldVars()){
			result.add(if_var);
		}
		return result;
	}
	protected void copy(StackLattice source, StackLattice dest) {
		source.copy(dest);
	}

	@Override
	protected void merge(StackLattice in1, StackLattice in2, StackLattice out) {
		out.clear();
		in1.intersection(in2, out);
		if(in1.equals(in2)){
			//end of an if structure
			out.removeLast();
		}
	}

	@Override
	protected StackLattice newInitialFlow() {
		return new StackLattice(refs);
	}

	@Override
	protected StackLattice entryInitialFlow() {
		return entryLattice.clone();
	}

	public StackLattice getFlowAtEnd() {
		return entryLattice.clone();
	}
}
