package pt.moth.dependencies.secondPhase;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import pt.moth.util.variables.IfConditionVar;



/**
 * This class represent a set of IfConditionVar. It allows us to simplify the
 * StackLattice code. With this class, a StackLattice has a Stack<SetIfVars>
 * instead of Stack<Set<IfConditionVar>>, that will simplify every methods
 * 
 * @author Vasco Pessanha
 * 
 */
public class SetIfVars implements Iterable<IfConditionVar> {
	protected Set<IfConditionVar> setOfVars;

	public SetIfVars() {
		this.setOfVars = new HashSet<IfConditionVar>();
	}

	/**
	 * Adds a new IfVar to the set.
	 * Returns true if this set did not already contain the specified element 
	 */
	public boolean add(IfConditionVar var) {
		return setOfVars.add(var);
	}

	/**
	 * Returns a clone of this SetIfVars
	 */
	public SetIfVars copy() {
		SetIfVars result = new SetIfVars();
		for (IfConditionVar v : setOfVars) {
			result.add((IfConditionVar) v.copy());
		}
		return result;
	}

	/**
	 * Returns the number of ifVars of this set
	 */
	public int size() {
		return setOfVars.size();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		for (IfConditionVar i : setOfVars) {
			result = prime * result + ((i == null) ? 0 : i.hashCode());
		}
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SetIfVars other = (SetIfVars) obj;
		if (setOfVars == null) {
			if (other.setOfVars != null)
				return false;
		} else if (setOfVars.size() != (other.setOfVars.size()))
			return false;
		for (IfConditionVar i : other.setOfVars) {
			if (!setOfVars.contains(i)) {
				return false;
			}
		}
		return true;
	}

	public String toString() {
		String r = "(";
		if (size() == 0)
			return r + ")";
		for (IfConditionVar i : setOfVars)
			r += i.toString() + ",";
		return r.substring(0, r.length() - 1) + ")";
	}

	@Override
	public Iterator<IfConditionVar> iterator() {
		return setOfVars.iterator();
	}

	public boolean isEmpty() {
		return setOfVars.isEmpty();
	}

	public void clear() {
		setOfVars.clear();
	}
}
