package pt.moth.instanceTypeAnalysis.firstPhase;

import pt.moth.inter.ForwardInterproceduralAnalysis;
import soot.Body;
import soot.SootMethod;
import soot.jimple.InvokeExpr;
import soot.toolkits.graph.BriefUnitGraph;
import soot.toolkits.graph.UnitGraph;

public class InterAnalysis extends ForwardInterproceduralAnalysis<ClassLattice> {
	
	public InterAnalysis() {
		super();
		analyze();
	}
	
	@Override
	protected ClassLattice analyzeMethod(Body activeBody, ClassLattice paramValues) {
		UnitGraph g = new BriefUnitGraph(activeBody);
		ClassLattice res = new IntraAnalysis(g, paramValues, this).getFlowAtEnd();
		return res;
	}

	@Override
	protected void copy(ClassLattice source, ClassLattice dest) {
		source.copy(dest);
	}

	@Override
	protected ClassLattice defaultParameterValues(SootMethod method) {
		return new ClassLattice();
	}

	@Override
	protected ClassLattice defaultReturnValue(SootMethod method) {
		return new ClassLattice();
	}

	@Override
	protected ClassLattice handleSkippedMethodInvocation(SootMethod callerMethod,
			ClassLattice out, InvokeExpr invokeExpr, SootMethod invokedMethod) {
		return (ClassLattice)out.clone();
	}

	@Override
	protected void merge(ClassLattice in1, ClassLattice in2, ClassLattice out) {
		in1.union(in2, out);
	}

	@Override
	protected ClassLattice newInitialFlow() {
		return new ClassLattice();
	}

	@Override
	protected void populateParameters(SootMethod callerMethod,
			SootMethod calleeMethod, InvokeExpr invokeExpr, ClassLattice out,
			ClassLattice parameter) {
		parameter.union(out);
	}

	@Override
	protected boolean shouldAnalyzeMethod(SootMethod method) {
		if (method.getDeclaringClass().isPhantom()) {
			return false;
		}
		else if (!method.hasActiveBody()) {
			return false;
		}
		return true;
	}

}
