package pt.moth.instanceTypeAnalysis.secondPhase;


import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import pt.moth.instanceTypeAnalysis.FieldClassesCollector;
import pt.moth.instanceTypeAnalysis.UnitsCollector;
import pt.moth.util.Pair;
import pt.moth.util.variables.GlobalVar;
import pt.moth.util.variables.LocalVar;


import soot.Local;
import soot.SootMethod;
import soot.Unit;
import soot.Value;
import soot.jimple.Constant;
import soot.jimple.FieldRef;
import soot.jimple.InvokeExpr;
import soot.jimple.MonitorStmt;
import soot.jimple.StaticFieldRef;
import soot.jimple.StaticInvokeExpr;
import soot.jimple.Stmt;
import soot.jimple.internal.AbstractBinopExpr;
import soot.jimple.internal.AbstractInstanceOfExpr;
import soot.jimple.internal.AbstractUnopExpr;
import soot.jimple.internal.JAddExpr;
import soot.jimple.internal.JArrayRef;
import soot.jimple.internal.JAssignStmt;
import soot.jimple.internal.JCastExpr;
import soot.jimple.internal.JGotoStmt;
import soot.jimple.internal.JIdentityStmt;
import soot.jimple.internal.JIfStmt;
import soot.jimple.internal.JInstanceFieldRef;
import soot.jimple.internal.JInterfaceInvokeExpr;
import soot.jimple.internal.JLookupSwitchStmt;
import soot.jimple.internal.JNewArrayExpr;
import soot.jimple.internal.JNewExpr;
import soot.jimple.internal.JNewMultiArrayExpr;
import soot.jimple.internal.JReturnStmt;
import soot.jimple.internal.JReturnVoidStmt;
import soot.jimple.internal.JSpecialInvokeExpr;
import soot.jimple.internal.JStaticInvokeExpr;
import soot.jimple.internal.JTableSwitchStmt;
import soot.jimple.internal.JThrowStmt;
import soot.jimple.internal.JVirtualInvokeExpr;
import soot.jimple.internal.JimpleLocal;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.graph.UnitGraph;
import soot.toolkits.scalar.ForwardFlowAnalysis;

public class IntraAnalysis extends ForwardFlowAnalysis<Unit, ParamLattice>{

	protected InterAnalysis interAnalysis;
	protected SootMethod thisMethod;
	
	/**
	 * This structure allows us to keep the set of readVariables
	 * that are stored in the outValue Lattice in the Return Stmt
	 */
	protected Map<LocalVar, FieldRef> readVariables;
	protected Map<LocalVar, String> newsExpressions;
	protected ParamLattice paramLattice;
	
	public IntraAnalysis(DirectedGraph<Unit> graph, ParamLattice paramValues, InterAnalysis interAnalysis) {
		super(graph);
		this.interAnalysis = interAnalysis;
		this.thisMethod = ((UnitGraph) graph).getBody().getMethod();

		this.readVariables = new HashMap<LocalVar, FieldRef>();
		this.newsExpressions = new HashMap<LocalVar, String>();
		this.paramLattice = paramValues;
		doAnalysis();
	}
	
	@Override
	protected void flowThrough(ParamLattice inValue, Unit unit, ParamLattice outValue) {
		
		inValue.copy(outValue);		
		if (((Stmt) unit).containsInvokeExpr()) {
			InvokeExpr expr = ((Stmt)unit).getInvokeExpr();
			SootMethod method = expr.getMethod();
			JimpleLocal jlocal = getInvokeExprLocal(expr);
			
			if (!interAnalysis.shouldAnalyzeMethod(method) && jlocal != null){
				GlobalVar var = null;
				LocalVar local = new LocalVar(thisMethod, jlocal);
				
				//Variavel é um parametro que tem uma variavel global
				if(paramLattice.contains(local)){
					var = paramLattice.getField(local);
				}
				//Metodo foi chamado numa variavel global
				else{
					FieldRef ref = readVariables.get(local);
					if(ref != null)
						var = new GlobalVar(ref);
					
				}
				Set<String> implementations = null;
				if(var != null){
					implementations = FieldClassesCollector.getImplementingClasses(var);
					if(implementations != null && implementations.size() > 0){
						UnitsCollector.putResult(unit, implementations);
					}
				}
			}
			else{
				if(method.hasActiveBody()){
					Iterator<Local> locals = method.getActiveBody().getLocals().iterator();
					int arg_num = 1;
					Local l;
					if(!(expr instanceof StaticInvokeExpr)){
						l = locals.next();
					}
					for(Object param : expr.getArgs()){
						l = locals.next();
						if(l instanceof JimpleLocal && param instanceof JimpleLocal){
							FieldRef ref = readVariables.get(new LocalVar(thisMethod,(JimpleLocal)param));
							if(ref != null){
								inValue.add(new Pair<LocalVar,GlobalVar>(new LocalVar(method,(JimpleLocal)l),new GlobalVar(ref)));
							}
						}
						arg_num ++;
					}
				}
				interAnalysis.handleMethodInvocation(thisMethod, inValue, (Stmt) unit);
			}
		}
		// This instruction is an assignment x = 4 or z = x
		else if (unit instanceof JAssignStmt) {
			Value leftop = ((JAssignStmt) unit).getLeftOp();
			Value rightop = ((JAssignStmt) unit).getRightOp();
			// Read(x)
			if (rightop instanceof JInstanceFieldRef || rightop instanceof StaticFieldRef) {
				if (leftop instanceof JimpleLocal) {
					FieldRef var = (FieldRef)rightop;
					LocalVar local = new LocalVar(thisMethod,(JimpleLocal)leftop);
					readVariables.put(local,var);
				}
				else {
					System.err.println("(IntraAnalysis) What Else???\nleftop = "+leftop+"\nrightop = "+rightop);
				}
			}
			// Write(x)
			else if (leftop instanceof JInstanceFieldRef) {}
			else if (leftop instanceof StaticFieldRef) {}
			else if(leftop instanceof JArrayRef){}
			else if (leftop instanceof JimpleLocal){
				if(rightop instanceof JNewExpr){}
				else if(rightop instanceof JCastExpr){}		//Ignorar casts
				else if(rightop instanceof JAddExpr){}	//Ignora somas..
				else if(rightop instanceof JimpleLocal){}	//Ignorar local = local...
				else if(rightop instanceof Constant){}	//Ignorar constantes
				else if(rightop instanceof JNewArrayExpr){}		//Ignorar new ?[]
				else if(rightop instanceof AbstractUnopExpr){}	//Ignorar expressoes de um arg (e.g. $r0.length())
				else if(rightop instanceof AbstractBinopExpr){}	//Ignorar expressoes de 2args (e.g. cmp)
				else if(rightop instanceof AbstractInstanceOfExpr){} //Ignorar expr "instanceof"
				else if(rightop instanceof JArrayRef){}				//Ignorar expressoes de arrays
				else if(rightop instanceof JNewMultiArrayExpr){}	//Ignorar New de arrys multiplos (tds duma vez)
				else{
					System.err.println("\nOther JAssignStmt Struture:");
					System.err.println("Left = " + leftop.getClass() + "("+ leftop + ")");
					System.err.println("Right = " + rightop.getClass() + "("+ rightop + ")\n");
				}
			}
			// Other JAssignStmt Struture
			else {
				System.err.println("\nOther JAssignStmt Struture:");
				System.err.println("Left = " + leftop.getClass() + "("+ leftop + ")");
				System.err.println("Right = " + rightop.getClass() + "("+ rightop + ")\n");
			}
		}
		else if(unit instanceof JReturnStmt || unit instanceof JReturnVoidStmt){}
		else if(unit instanceof JIdentityStmt){}
		else if(unit instanceof JIfStmt){}//This can identify a if with local vars (e.g. if($r0 == null))
		else if(unit instanceof JThrowStmt){}//Throw exception with localVariable (e.g. throw $r0)
		else if(unit instanceof JGotoStmt){}//JGotoStmt used, for instances, in loops
		else if(unit instanceof JTableSwitchStmt){}//JTableSwitchStmt used
		else if(unit instanceof MonitorStmt){}//Monitor Statement
		else if(unit instanceof JLookupSwitchStmt){}//JLookupSwitchStmt used
		// Others..
		else {
			System.err.println("("+thisMethod.getName()+")Other class : (in "+unit.getClass()+") "+unit);
		}
//		System.out.println("("+thisMethod.getDeclaringClass()+","+thisMethod.getName() + ") " + unit + "("+unit.getClass()+") - "+inValue+","+outValue);
		
	}
	
	//Returns the local variable (Base) of a InvokeExpr.
	//E.g. $r0.getX() returns "$r0"
	private JimpleLocal getInvokeExprLocal(InvokeExpr expr){
		if(expr instanceof JVirtualInvokeExpr && ((JVirtualInvokeExpr)expr).getBase() instanceof JimpleLocal){
			return (JimpleLocal)((JVirtualInvokeExpr)expr).getBase();
		}
		else if(expr instanceof JInterfaceInvokeExpr && ((JInterfaceInvokeExpr)expr).getBase() instanceof JimpleLocal){
			return (JimpleLocal)((JInterfaceInvokeExpr)expr).getBase();
		}
		else if(expr instanceof JSpecialInvokeExpr && ((JSpecialInvokeExpr)expr).getBase() instanceof JimpleLocal){
			return (JimpleLocal)((JSpecialInvokeExpr)expr).getBase();
		}
		else if(expr instanceof JStaticInvokeExpr){}//Ignore!		
		else{
			System.err.println("Other InvokeExpr class: " + (expr.getClass()));
		}
		return null;
	}
	
	protected void copy(ParamLattice source, ParamLattice dest) {
		source.copy(dest);
	}

	@Override
	protected ParamLattice entryInitialFlow() {
		return new ParamLattice();
	}

	@Override
	protected void merge(ParamLattice in1, ParamLattice in2, ParamLattice out) {
		in1.union(in2, out);
	}

	@Override
	protected ParamLattice newInitialFlow() {
		return new ParamLattice();
	}

	public ParamLattice getFlowAtEnd() {
		ParamLattice res = newInitialFlow();

		// merge the flowsets of all the method's exit-points
		List<Unit> tails = this.graph.getTails();
		for (Unit unit : tails) {
			ParamLattice sep1 = getFlowAfter(unit);
			if (sep1 == null) { // this method was not analyzed
				return res;
			}
			ParamLattice tmp = newInitialFlow();
			sep1.union(res, tmp);
			res = tmp;
		}
		return res;
	}
}
