package pt.moth.processAnalysis;


import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import pt.moth.inter.MethodUtils;

import soot.Kind;
import soot.MethodOrMethodContext;
import soot.SootMethod;
import soot.jimple.toolkits.callgraph.CallGraph;
import soot.jimple.toolkits.callgraph.Edge;

/**
 * This class compute the set of processes that can execute. For each process we
 * associate all the atomic methods. This will be useful to compute the views of
 * the processes
 * 
 * @author Vasco Pessanha
 */
public class ProcessGenerator {
	
	//Methods that have been already visited
	//Could be a Set<> but the search operation of a Map is faster
	private static Map<String, String> visitedMethods = new HashMap<String, String>();

	// Map of all processes identified by their name
	private static Map<String, Process> processes = new HashMap<String, Process>();
	// Set of the classes that represent threads and were already considered as
	// a process. If we have a lot of calls of threads of a especific class we
	// only want one process to be evaluated
	private static Set<String> checkedclasses = new HashSet<String>();

	public static Set<Process> generateProcesses(CallGraph cg) {
		Process main = new Process("Process 1", "Process generated by the Main");
		processes.put("Process 1", main);
		Iterator<MethodOrMethodContext> x = cg.sourceMethods();
		MethodOrMethodContext next;

		while (x.hasNext()) {
			next = x.next();
			if (next instanceof SootMethod) {
				SootMethod sm = (SootMethod) next;
				if (sm.getSignature().contains("void main(java.lang.String[])")) {
					analyseEdges(cg, sm, main.getName());
					break;
				}
			} else {
				System.out.println("This is not a SootMethod: " + next);
			}
		}
		return new HashSet<Process>(processes.values());
	}

	private static void analyseEdges(CallGraph cg, SootMethod method, String p) {
		Edge edge;
		Kind kind;
		Iterator<Edge> iterator = cg.edgesOutOf(method);
		while (iterator.hasNext()) {
			edge = iterator.next();
			kind = edge.kind();
			String target = "";
			if(edge.srcStmt() != null && edge.srcStmt().containsInvokeExpr()){
				target = edge.srcStmt().getInvokeExpr().toString();
				target = kind.toString()+" "+target.substring(target.indexOf('<')+1,target.indexOf('>'));
			}
			else{}
			boolean visited = (visitedMethods.get(p+target) != null);
			if (!visited) {
				visitedMethods.put(p+target, p+target);
				// If we have an Thread.run() invoke than we can have another process!
				if (kind.equals(Kind.THREAD)) {
					if (edge.getTgt() instanceof SootMethod) {
						boolean isNew = checkedclasses.add(target);
						if (isNew) {
							String name = "Process " + (processes.size() + 1);
							processes.put(name, new Process(name, target));
							// Analyse edges of the method
							analyseEdges(cg, (SootMethod) edge.getTgt(), name);
						} else {
							// System.out.println("Already have this Class.run()'s process");
						}
					} else {
						System.err.println("ERROR! (ProcessGenerator) - Caught a Thread.run() but the target was not a SootMethod");
					}
				} else if (kind.equals(Kind.SPECIAL) || kind.equals(Kind.STATIC) || kind.equals(Kind.VIRTUAL)) {
					if (edge.getTgt() instanceof SootMethod) {
						SootMethod sm = (SootMethod) edge.getTgt();
						// This is an atomic block!
						if (MethodUtils.methodIsAtomic(sm)) {
							processes.get(p).addMethod(sm);
						}
						// Is not atomic, keep looking..
						else {
							analyseEdges(cg, sm, p);
							// System.out.println("ANALISAR "+ sm);
						}
					} else {
						System.out
								.println("ERROR! (ProcessGenerator) - Caught a SPECIAL/STATIC but the target was not a SootMethod");
					}
				} else {
//					System.out.println("Outro tipo de Edge\nTarget: "+edge.getTgt());
//					System.out.println("Kind: "+edge.kind());
//					System.out.println("STMT: "+edge.srcStmt()+"\n");
				}
			}
		}

	}
	
	public static Process getProcess(String name) {
		return processes.get(name);
	}
}
