package pt.moth.sensors;

import java.util.Set;

import pt.moth.util.conflicts.Conflict;


/**
 * This Interface represents a general Sensor. A sensor is a component that can
 * detect high-level anomalies. Our approach is composed by a series of sensors
 * that detect specific types of sensors. The sets of conflicts detected by each
 * Sensor don't have to be disjunct, since the conflicts are merged before
 * presented to the user.
 * 
 * @author Vasco Pessanha
 * 
 */
public interface Sensor {
	/**
	 * Push all the view information needed by the sensor. That information
	 * consists in all the information about processes and their views, and is
	 * kept in the ViewsCollector class
	 */
	public void pushViewInformation();
	
	/**
	 * Push all the information about dependencies needed by the sensor.
	 * 
	 * @param ViewsCollector that has all the
	 * information about processes and their views
	 */
	public void pushDepInformation();

	
	/**
	 * Returns all the conflicts detected by the given sensor
	 * 
	 * @return the set of conflicts detected by the given sensor
	 */
	public Set<Conflict> getConflicts();
}
