package pt.moth.sensors;

import java.util.HashMap;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import pt.moth.processAnalysis.Process;
import pt.moth.util.Access;
import pt.moth.util.IAccess;
import pt.moth.util.View;
import pt.moth.util.conflicts.Conflict;
import pt.moth.util.conflicts.RwWConflict;
import pt.moth.util.variables.GlobalVar;
import pt.moth.viewsAnalysis.ViewsCollector;

@Deprecated
public class SingleVariableSensor implements Sensor{
	public final boolean EXCLUDERwRW = false;
	// List of all the processes
	private Set<Process> processes = new HashSet<Process>();
	
	// For each Process, these atributes keep the views (Read and Write)
	// of that process
	private Map<String, List<View>> read_views;
	private Map<String, List<View>> write_views;
	
	
	public SingleVariableSensor(){
		read_views = new HashMap<String, List<View>>();
		write_views = new HashMap<String, List<View>>();
	}
	
	@Override
	public void pushViewInformation() {
		//For each process...
		for(Entry<String, List<View>> e : ViewsCollector.getReadViews()){
			read_views.put(e.getKey(),e.getValue());
		}
		for(Entry<String, List<View>> e : ViewsCollector.getWriteViews()){
			write_views.put(e.getKey(),e.getValue());
		}
		processes = ViewsCollector.getProcesses();
	}
	
	@Override
	public Set<Conflict> getConflicts() {
		boolean sameSize = (read_views.size() == write_views.size())
				&& (read_views.size() == processes.size());
		Set<Conflict> conflicts = new HashSet<Conflict>();
		if (!sameSize) {
			System.err
					.println("Error! (SingleVariableSensor) Every of this atributes should have the same size (equals to the number of the processes)");
		} else {
			//the size could be obtained through another attribute
			Process[] array = new Process[processes.size()];
			int pos = 0;
			// Store in the array
			for (Process p : processes) {
				array[pos] = p;
				pos++;
			}
			// Match every processor with the other
			for (int i = 0; i < array.length; i++) {
				for (int j = i; j < array.length; j++) {
					getConflicts(conflicts, array[i], array[j]);
					if (i != j) {
						getConflicts(conflicts, array[j], array[i]);
					}
				}
			}
		}
		return conflicts;
	}
	
	/**
	 * This Method computes all the RwW conflicts If p2 reads and then writes a
	 * given variable, and p1 writes on that variable, then we have a conflict
	 * 
	 * @param conflicts Set of conflicts on which we will add this type of conflicts
	 * @param p1 Process that may read a variable and then, based on that value,
	 *            write on it in another transaction
	 * @param p2 Process that writes in that variable between the first two transactions
	 */
	private void getConflicts(Set<Conflict> conflicts, Process p1,
			Process p2) {
		List<View> p1_ViewsW = write_views.get(p1.getName());
		List<View> p2_ViewsR = read_views.get(p2.getName());
		List<View> p2_ViewsW = write_views.get(p2.getName());

		// Methods were the variables are read/written
		Map<GlobalVar, View> readMethod = new HashMap<GlobalVar, View>();
		Map<GlobalVar, View> writeMethod = new HashMap<GlobalVar, View>();

		// Variables that were Read and written by Process2.
		Set<GlobalVar> potencialStaleValues= new HashSet<GlobalVar>();
		getPossibleStaleValues(p2_ViewsR, p2_ViewsW, readMethod, writeMethod,
				potencialStaleValues);

		boolean found;
		for (GlobalVar v : potencialStaleValues) {
			found = false;
			for (View view : p1_ViewsW) {
				for (IAccess a : view.getAccesses().toList()) {
					if (a instanceof Access && v.equals(a.getVariable())) {
						found = true;
						conflicts.add(new RwWConflict(p1, p2, v, readMethod
								.get(v), writeMethod.get(v), view));
						break;
					}
				}
				if (found) {
					break;
				}
			}
		}
	}

	private void getPossibleStaleValues(List<View> p2_ViewsR,
			List<View> p2_ViewsW, Map<GlobalVar, View> readMethod,
			Map<GlobalVar, View> writeMethod,
			Set<GlobalVar> potencialThreats) {
		// For each variable read, 'readVariables' gives all the methods that
		// read that variable
		Map<GlobalVar, Set<View>> readVariables = new HashMap<GlobalVar, Set<View>>();
		for (View v : p2_ViewsR) {
			for (IAccess a : v.getAccesses().toList()) {
				if (a instanceof Access && ((Access) a).isRead()/* && !((Access) a).closedBullet()*/) {
					// Is potencial if it is read in another method
					if (readVariables.get(a.getVariable()) == null) {
						readVariables.put(new GlobalVar((GlobalVar)a.getVariable()),
								new HashSet<View>());
					}
					readVariables.get(a.getVariable()).add(v);
				}
			}
		}
		// readVariables has the read variables (that were not smashed in that
		// block)
		for (View v : p2_ViewsW) {
			for (IAccess a : v.getAccesses().toList()) {
				if (a instanceof Access && ((Access) a).isWrite() 
						/*&& (!((Access) a).closedBullet()) || !EXCLUDERwRW*/) {
					GlobalVar var = new GlobalVar((GlobalVar)a.getVariable());
					Set<View> methods = readVariables.get(var);
					if (methods != null) {
						for (View m : methods) {
							readMethod.put(var, m);
							writeMethod.put(var, v);
							potencialThreats.add(var);
							break;
						}
					}
				}
			}
		}
	}

	@Override
	public void pushDepInformation() {
		//Does not need dependency information		
	}

}
