package pt.moth.util;

import pt.moth.util.variables.Variable;

/**
 * This class represent an access to a given variable (class' field)
 * An Access is a triple (alfa,v,b) where:
 * 	- alfa is {r,w} (read, write)
 *  - v is the variable accessed
 * 
 * These accesses are described in PADTAD 2011
 * @author Vasco Pessanha
 */
public class Access implements IAccess {
	
	protected Variable variable;
	// 0 - read, 1 - write
	protected boolean type;	
	
	public Access(boolean iswrite, Variable jinstance/*, boolean closedBullet*/) {
		this.type = iswrite;
		this.variable = jinstance;
	}
	
	public Variable getVariable(){
		return variable;
	}

	public String getName() {
		return variable.toString();
	}
	
	public boolean isWrite(){
		return type;
	}
	
	public boolean isRead(){
		return !type;
	}
	
	@Override
	public int hashCodeIgnoreType() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((variable == null) ? 0 : variable.hashCode());
		return result;
	}

	@Override
	public boolean equalsIgnoreType(IAccess obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Access))
			return false;
		Access other = (Access) obj;
		return (variable.equals(other.variable));
	}

	public boolean equalsWithoutBullet(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if(!(obj instanceof Access))
			return false;
		Access a = (Access) obj;
		return (variable.equals(a.variable) && type == a.type);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (type ? 1231 : 1237);
		result = prime * result
				+ ((variable == null) ? 0 : variable.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Access))
			return false;
		Access other = (Access) obj;
		if (type != other.type)
			return false;
		if (variable == null) {
			if (other.variable != null)
				return false;
		} else if (!variable.equals(other.variable))
			return false;
		return true;
	}
	public String toString(){
		return "("+(type?"w":"r")+","+variable.toString()+")";
	}
	
	@Override
	public Access copy() {
		return new Access(type, variable);
	}
}