package pt.moth.util.conflicts;

/**
 * This Class defines the possible set of conflict types.
 * 
 * @author Vasco Pessanha
 *
 */
public class ConflictType {
	
	public static final int WrWType = 1;
	public static final int WwWType = 2;
	public static final int RwRType = 3;
	public static final int ARTHOType = 4;
//	public static final int RwWType = 5;
	public static final int DepType = 5;
	
	private final String WrWString = "WrW";
	private final String WwWString = "WwW";
	private final String RwRString = "RwR";
	private final String ARTHOSTRING = "Artho";
//	private final String RwWString = "RwW Conflict";
	private final String DepString = "Dep's Conflict";
	
	public static final ConflictType WrW = new ConflictType(WrWType);
	public static final ConflictType WwW = new ConflictType(WwWType);
	public static final ConflictType RwR = new ConflictType(RwRType);
	public static final ConflictType ARTHO = new ConflictType(ARTHOType);
//	public static final ConflictType RwW = new ConflictType(RwWType);
	public static final ConflictType DEP = new ConflictType(DepType);

	
	protected int type;
	public ConflictType(int type){
		isValid(type);
		this.type = type;
	}
	private void isValid(int inType) {
		if(inType != WrWType && inType != WwWType && 
			inType != RwRType && inType != ARTHOType && 
			/*inType != RwWType &&*/ inType != DepType){
			
			System.err.println("ERROR (ConflictType) - tried to introduced an invalid type ("+inType+")");
			System.exit(-1);
		}
	}
	
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + type;
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConflictType other = (ConflictType) obj;
		if (type != other.type)
			return false;
		return true;
	}
	public String toString(){
		if(this.equals(WrW))
			return WrWString;
		else if(this.equals(WwW))
			return WwWString;
		else if(this.equals(RwR))
			return RwRString;
		else if(this.equals(ARTHO))
			return ARTHOSTRING;
//		else if(this.equals(RwW))
//			return RwWString;
		else if(this.equals(DEP))
			return DepString;
		else
			return "WRONG TYPE";
	}
}
