package pt.moth.util.conflicts;

import java.util.Set;

import pt.moth.processAnalysis.Process;
import pt.moth.util.View;


/**
 * This Class represent a Conflict between a set o views and a MaximalView. A
 * conflict occurs when we the views of a process are included in a certain
 * Maximal view and don't form a chain
 * 
 * The 'equals' and 'hashCode' methods are only relative to the views since a
 * set of problematic views can be conflictuous with a large set of blocks
 * representing the same conflict
 * 
 * @author Vasco Pessanha
 * 
 */
public class ViewConflict extends Conflict{
	
	protected Set<View> p2Views;
	protected View p1MaxView;
	
	public ViewConflict (Process p1, Process p2, Set<View> views, View maximalView, ConflictType type){
		this.p1 = p1;
		this.p2 = p2;
		this.p2Views = views;
		this.p1MaxView = maximalView;
		this.type = type;
	}
	public Set<View> getViews() {
		return p2Views;
	}
	public void setViews(Set<View> views) {
		this.p2Views = views;
	}
	public View getMaximalView() {
		return p1MaxView;
	}
	public void setMaximalView(View maximalView) {
		this.p1MaxView = maximalView;
	}
	public String toString(){
		String result = "ViewConflict("+type.toString()+") between " + p1.getName() + " and "
				+ p2.getName() + "!!!\nMethod: '"
				+ p1MaxView.getMethod().getName() + "()'\nVS\nMethods: ";
		for (View v : p2Views) {
			result += "'" + v.getMethod().getName() + "()',";
		}
		return result.substring(0,result.length()-1);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((p2Views == null) ? 0 : p2Views.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ViewConflict))
			return false;
		ViewConflict other = (ViewConflict) obj;
		if (p2Views == null) {
			if (other.p2Views != null)
				return false;
		} else if (!p2Views.equals(other.p2Views))
			return false;
		else if(type != other.type)
			return false;
		return true;
	}
}
