package pt.moth.util.conflicts;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import pt.moth.util.View;


public class ViewGroup implements IConflictGroup {
	
	protected List<ViewConflict> conflicts;

	public ViewGroup(ViewConflict conflict) {
		conflicts = new ArrayList<ViewConflict>();
		conflicts.add(conflict);
	}
	
	
	@Override
	public boolean addConflict(Conflict conflict) {
		if(!(conflict instanceof ViewConflict)){
			return false;
		}
		ViewConflict head = conflicts.get(0);
		ViewConflict c = (ViewConflict)conflict;
		
		if (!head.p1MaxView.getMethod().equals(c.p1MaxView.getMethod())) {
			return false; // Maximal views of different method
		}
		if (compareSetsIgnoreCase(head.p2Views, c.p2Views)) {
			conflicts.add(c);
			return true;
		}
		return false;
	}

	private boolean compareSetsIgnoreCase(Set<View> v1, Set<View> v2) {
		if (v1.size() != v2.size())
			return false;
		for (View v : v1) {
			boolean missed = true;
			for (View other : v2) {
				if (v.equalsIgnoreType(other)) {
					missed = false;
					break;
				}
			}
			if (missed) {
				return false;
			}
		}
		return true;
	}
	
	@Override
	public String toString() {
		Iterator<ViewConflict> it = conflicts.iterator();
		ViewConflict head = (ViewConflict) it.next();

		Set<ConflictType> types = new HashSet<ConflictType>();
		types.add(head.getConflictType());
		while (it.hasNext()) {
			types.add(it.next().getConflictType());
		}
		String result = "ViewConflict(";

		for (ConflictType type : types) {
			result += type.toString() + ", ";
		}
		result = result.substring(0, result.length() - 2);
		result += ")between " + head.p1.getName() + " and " + head.p2.getName()
				+ "!!!\nMaxView = '" + head.p1MaxView.getMethod().getName()
				+ "()'\nViews = {";
		for (View v : head.p2Views) {
			result += v.getMethod().getName() + "(),";
		}
		return result.substring(0, result.length() - 1) + "}";
	}
}
