package pt.moth.util.graphs;

/**
 * This class represents a graph's edge with a origin and a destination
 * "(origin -> dest)"
 * 
 * @author Vasco Pessanha
 * @param <E> - vertices' type
 */
public class Edge<E> implements IEdge<E>{
	protected E origin;
	protected E dest;
	
	public Edge(E origin, E dest){
		this.origin = origin;
		this.dest = dest;
	}
	
	/**
	 * Returns the edge's origin
	 * @return edge's origin
	 */
	public E getOrigin(){
		return origin;
	}
	/**
	 * Returns the edge's dest
	 * @return edge's dest
	 */
	public E getDest(){
		return dest;
	}
	
	/**
	 * Prints this edge's description
	 */
	public String toString(){
		return "("+origin.toString()+" -> "+dest.toString()+")";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((origin == null) ? 0 : origin.hashCode());
		result = prime * result + ((dest == null) ? 0 : dest.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Edge))
			return false;
		@SuppressWarnings("unchecked")
		Edge<E> other = (Edge<E>) obj;
		if (origin == null) {
			if (other.origin != null)
				return false;
		} else if (!origin.equals(other.origin))
			return false;
		if (dest == null) {
			if (other.dest != null)
				return false;
		} else if (!dest.equals(other.dest))
			return false;
		return true;
	}
	
	/**
	 * Returns a clone of this edge
	 */
	public Edge<E> copy(){
		return new Edge<E>(origin,dest);
	}
}
