package pt.moth.util.variables;

import soot.SootMethod;
import soot.jimple.Stmt;
import soot.jimple.internal.JimpleLocal;

/**
 * This Class represents a local variable that is present in a given if condition.
 * We need this kind of variables to find dependencies between views:
 * if(result(ViewA)...){
 * 		ViewB
 * }
 * 
 * ViewA  -->  ViewB
 * 		 Ifcond
 * 
 * @ifStmt, if Stmt where the variable was used
 * @global, Soot Object that has information about the global variable
 * @method, Method's name
 * 
 * @author Vasco Pessanha
 * 
 */
public class IfLocalVar implements IfConditionVar{

	private String name;
	private SootMethod method;
	private JimpleLocal local;
	private Stmt ifStmt;

	public IfLocalVar(IfLocalVar v) {
		this.name = v.name;
		this.method = v.method;
		this.local = v.local;
		this.ifStmt = v.ifStmt;
	}
	
	public IfLocalVar(SootMethod method, JimpleLocal ifLocal, Stmt ifStmt) {
		this.method = method;
		this.local = ifLocal;
		this.name = method.getName()+"."+local.getName()+"(if)";
		this.ifStmt = ifStmt;
	}
	public String toString(){
		return name;
	}
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public JimpleLocal getJimpleLocal() {
		return local;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((local == null) ? 0 : local.hashCode());
		result = prime * result
				+ ((method == null) ? 0 : method.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof IfLocalVar))
			return false;
		IfLocalVar other = (IfLocalVar) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public IfLocalVar copy() {
		return new IfLocalVar(method, local, ifStmt);
	}

	@Override
	public Stmt getIfStmt() {
		return ifStmt;
	}

	@Override
	public SootMethod getSootMethod() {
		return method;
	}
}
