package pt.moth.util.variables;

import soot.SootMethod;

/**
 * This class represents a special variable only used in native methods. Since a
 * native method can both read and write on parameters or other fields from the
 * class, we have to assume a dependency between the paramteres and those
 * globalVars. However, instead of that, we create a special globalVar of that
 * native method and connect every args to that variable.
 * 
 * @author Vasco Pessanha
 * 
 */
public class NativeGlobalVar implements IGlobalVar{

	// Native Method that was called
	protected SootMethod nativeMethod;
	// Method that called the native method
	protected SootMethod callerMethod;
	// Atomic method that called all this method 
	protected SootMethod atomicMethod;
	// This "variable"'s name
	protected String name;
	public NativeGlobalVar(SootMethod nativeMethod, SootMethod caller, SootMethod atomic){
		this.nativeMethod = nativeMethod;
		this.callerMethod = caller;
		this.atomicMethod = atomic;
		this.name = generateName();
	}
	
	private String generateName() {
		return "NativeGlobarVar ("+nativeMethod.getName()+","+atomicMethod.getName()+")";
	}

	public String getName() {
		return name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NativeGlobalVar other = (NativeGlobalVar) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return name;
	}

	@Override
	public NativeGlobalVar copy() {
		return new NativeGlobalVar(nativeMethod,callerMethod,atomicMethod);
	}

	@Override
	public SootMethod getSootMethod() {
		return atomicMethod;
	}

	public SootMethod getAtomicMethod() {
		return atomicMethod;
	}
}
