package pt.moth.util.variables;

import pt.moth.util.variables.Variable;
import soot.SootMethod;

/**
 * This Class represents a local variable. Despite the fact that the methods
 * equals and hashCode are implemented using only the name atribute, we keep the
 * Soot Objects for future using of their information
 * The name of a local variable consists in: (variable's method + "." + its name)
 * The atributes are:
 * 
 * @name, is the name of this variable ("method.name")
 * @local, Soot Object that has information about the local variable
 * @method, Method's name
 * 
 * @author Vasco Pessanha
 * 
 */
public class ParamVar implements Variable{

	private String name;
	private SootMethod method;
	private int num;

	public ParamVar(ParamVar v) {
		this.name = v.name;
		this.method = v.method;
		this.num = v.num;
	}
	
	public ParamVar(SootMethod method, int num) {
		this.method = method;
		this.name = method.getSignature()+".param"+num;
		this.num = num;
	}
	public String toString(){
		return name;
	}
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((method == null) ? 0 : method.getSignature().hashCode());
		result = prime * result + num;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ParamVar))
			return false;
		ParamVar other = (ParamVar) obj;
		if (method == null) {
			if (other.method != null)
				return false;
		} else if (!method.equals(other.method))
			return false;
		return num == other.num;
	}

	@Override
	public ParamVar copy() {
		return new ParamVar(method, num);
	}

	public SootMethod getSootMethod() {
		return method;
	}
}
