package pt.moth.viewsAnalysis;

import pt.moth.inter.ForwardInterproceduralAnalysis;
import pt.moth.inter.MethodUtils;
import pt.moth.xmlAnnotations.AnnotationsParser;
import soot.Body;
import soot.SootMethod;
import soot.Unit;
import soot.jimple.InvokeExpr;
import soot.tagkit.AnnotationTag;
import soot.tagkit.Tag;
import soot.tagkit.VisibilityAnnotationTag;
import soot.toolkits.graph.BriefUnitGraph;
import soot.toolkits.graph.UnitGraph;

public class InterAnalysis extends ForwardInterproceduralAnalysis<Lattice> {
	
	private AnnotationsParser methodAnnotations;	
	
	public InterAnalysis(AnnotationsParser methodAnnotations) {
		super();
		this.methodAnnotations = methodAnnotations; 
		analyze();
	}
	
	public static boolean isAnnotatedWith(SootMethod method, String anotClass) {
		for (Tag t : method.getTags()) {
			if (t instanceof VisibilityAnnotationTag) {
				for (AnnotationTag at : ((VisibilityAnnotationTag)t).getAnnotations()) {
					if (at.getType().equals(anotClass)) {
						return true;							
					}
				}
			}
		}
		return false;
	}
	
	@Override
	protected Lattice analyzeMethod(Body activeBody, Lattice paramValues) {
		UnitGraph g = new BriefUnitGraph(activeBody);
		//Method with "Atomic" Tag
		if (MethodUtils.methodIsAtomic(activeBody.getMethod())) {
			Lattice res = new IntraAnalysisAtomic(g, this, methodAnnotations).getFlowAtEnd();
			SootMethod method = activeBody.getMethod();
			ResultsCollector.putResult(method, res);
			return res;
		}
		
		//Method without "Atomic" Tag
		else {
			//Called by a Atomic Method
			if (paramValues.equals(Lattice.ATOMIC_LATTICE)) {
				Lattice x =  new IntraAnalysisAtomic(g, this, methodAnnotations).getFlowAtEnd();
				return x;
			}
			//Called by non Atomic Methods
			else {
				return new IntraAnalysisNonAtomic<Unit>(g, this).getFlowAtEnd();				
			}
		}
		
	}

	@Override
	protected void copy(Lattice source, Lattice dest) {
		source.copy(dest);
	}

	@Override
	protected Lattice defaultParameterValues(SootMethod method) {
		return new Lattice();
	}

	@Override
	protected Lattice defaultReturnValue(SootMethod method) {
		return new Lattice();
	}

	@Override
	protected Lattice handleSkippedMethodInvocation(SootMethod callerMethod,
			Lattice out, InvokeExpr invokeExpr, SootMethod invokedMethod) {
		return (Lattice)out.clone();
	}

	@Override
	protected void merge(Lattice in1, Lattice in2, Lattice out) {
		in1.union(in2, out);
	}

	@Override
	protected Lattice newInitialFlow() {
		return new Lattice();
	}

	@Override
	protected void populateParameters(SootMethod callerMethod,
			SootMethod calleeMethod, InvokeExpr invokeExpr, Lattice out,
			Lattice parameter) {
		parameter.union(out);
	}

	@Override
	protected boolean shouldAnalyzeMethod(SootMethod method) {
		if (method.getDeclaringClass().isPhantom()) {
			return false;
		}
		else if (!method.hasActiveBody()) {
			return false;
		}
		return true;
	}

}
