package pt.moth.viewsAnalysis;

import java.util.HashMap;
import java.util.Map;

import pt.moth.util.View;

import soot.SootMethod;

/**
 * This class allows us to store each result while evaluating each method.
 * A "result" is the set of accesses of one specific method.
 *  
 * @author Vasco Pessanha
 *
 */
public class ResultsCollector {
	private static Map<String,Lattice> results = new HashMap<String, Lattice>();
	private static boolean hasResults = false; 
	/**
	 * Store the lattice of that method.
	 * If the method has already a lattice, the two are merged
	 * @param method - method whose view is being stored
	 * @param res - View of the method
	 */
	public static void putResult(SootMethod method, Lattice res) {
		Lattice copy = new Lattice();
		res.copy(copy);
		results.put(method.getSignature(), copy);
		hasResults = true;
	}
	
	/**
	 * Allows us to know if we have already computed the view analysis
	 */
	public static boolean hasResults(){
		return hasResults;
	}
	
	/**
	 * Returns the Lattice of a method
	 * @param methodSig signature of the method whose Lattice we want to see
	 * @return lattice of that method
	 */
	public static Lattice getLattice(String methodSig){
		Lattice lat = results.get(methodSig);
		Lattice toReturn = new Lattice();
		if(lat != null){
			lat.copy(toReturn);
			return toReturn;
		}
		return null;
	}
	
	/**
	 * Returns the View of a method
	 * @param methodSig signature of the method whose Lattice we want to see
	 * @return view of that method
	 */
	public static View getView(SootMethod method){
		Lattice toReturn = getLattice(method.getSignature());
		if(toReturn != null){
			return new View(toReturn, method);
		}
		return null;
	}
}
