package pt.moth.xmlAnnotations;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import soot.SootMethod;

/**
 * This class has just one method that receives a native method and annotates
 * it. We assume the worst possible cenario! Therefore, we assume that the
 * methods reads and writes in every parameter and in the main object
 * 
 * @author Vasco Pessanha
 */
public class NativeAnnotator {

	private static final String FILE_NAME = "methods.xml";

	public static void annotate(SootMethod nativeMethod, boolean hasObject) {
		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(FILE_NAME);

			MethodName methodName = new MethodName(nativeMethod);

			// Get the root element
			Node classes = doc.getFirstChild();

			// Get our Class
			Element ourClass = doc.getElementById(methodName.getObjClass());
			if (ourClass == null) {
				ourClass = doc.createElement("automatic_class");
				ourClass.setAttribute("id", methodName.getObjClass());
			}
			// create method
			Element method = doc.createElement("method");
			method.setAttribute("id", methodName.getMethodSignature());

			// Create read and write annotations
			Element reads = doc.createElement("reads");
			Element writes = doc.createElement("changes");
			reads.setTextContent("this");
			writes.setTextContent("this");

			// create params
			int size = nativeMethod.getParameterCount();
			Element[] params = new Element[size * 2];
			for (int i = 0; i < size; i++) {
				int j = i * 2;
				params[j] = doc.createElement("changes");
				params[j + 1] = doc.createElement("reads");

				params[j].setTextContent(i + "");
				params[j + 1].setTextContent(i + "");
			}

			// append these nodes to method node
			if(hasObject){
				method.appendChild(reads);
				method.appendChild(writes);
			}
			for (int i = 0; i < size * 2; i++) {
				method.appendChild(params[i]);
			}

			// append method node to class node

			ourClass.appendChild(method);
			// append a new class node to classes
			classes.appendChild(ourClass);

			// write the content into xml file
			TransformerFactory transformerFactory = TransformerFactory
					.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(new File(FILE_NAME));
			transformer.transform(source, result);

		} catch (Exception e) {// Catch exception if any
			System.err.println("(NativeAnnotator) Error! " + e.getMessage());
		}
	}
}