package pt.moth;

import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;



import pt.moth.arthoApproach.ArthoSensor;
import pt.moth.arthoApproach.ArthoViewsCollector;
import pt.moth.dependencies.VarDepCollector;
import pt.moth.dependencies.thirdPhase.ViewDepCollector;
import pt.moth.instanceTypeAnalysis.UnitsCollector;
import pt.moth.processAnalysis.Process;
import pt.moth.processAnalysis.ProcessGenerator;
import pt.moth.sensors.DepSensor;
import pt.moth.sensors.Sensor;
import pt.moth.sensors.ViewConsistencySensor;
import pt.moth.util.conflicts.Conflict;
import pt.moth.util.conflicts.DepConflict;
import pt.moth.util.conflicts.DepGroup;
import pt.moth.util.conflicts.IConflictGroup;
import pt.moth.util.conflicts.ViewConflict;
import pt.moth.util.conflicts.ViewGroup;
import pt.moth.viewsAnalysis.ViewsCollector;
import pt.moth.xmlAnnotations.AnnotationsParser;
import pt.moth.xmlAnnotations.MethodName;
import pt.moth.xmlAnnotations.NativesCollector;



import soot.Scene;
import soot.SceneTransformer;
import soot.Unit;
import soot.jimple.toolkits.callgraph.CallGraph;
 
public class AnalysisTransformer extends SceneTransformer {
	
	//XML File with method's annotations
	private final String DEFAULT_XML_FILE = "res/methods.xml";
	private final Integer SENSORS_NUMBER = 2;
	private AnnotationsParser methodAnnotations;
	
	public AnalysisTransformer() {
		methodAnnotations = new AnnotationsParser(DEFAULT_XML_FILE);
	}
	public AnalysisTransformer(String xml_file) {
		methodAnnotations = new AnnotationsParser(xml_file);
	}

	private void callInstanceTypeAnalysis(){
		new pt.moth.instanceTypeAnalysis.firstPhase.InterAnalysis();

		new pt.moth.instanceTypeAnalysis.secondPhase.InterAnalysis();
	}
	private void callViewsAnalysis(){
		new pt.moth.viewsAnalysis.InterAnalysis(methodAnnotations);
	}
	private void callDependencyAnalysis(){
		new pt.moth.dependencies.firsPhase.InterAnalysis();
		System.out.println("FINAL DA PRIMEIRA!!!!!!!!!!!!!!!!!!!!!!! \n");
		new pt.moth.dependencies.secondPhase.InterAnalysis();
		System.out.println("FINAL DA SEGUNDA!!!!!!!!!!!!!!!!!!!!!!! \n");
		VarDepCollector.pruneGraph();
		System.out.println("PRUNNING DONE!!!!!!!!!!!!!!!!!!!!!!! \n");
		ViewDepCollector.computeViewDeps();
		System.out.println(ViewDepCollector.printDeps());
	}
	@Override
	protected void internalTransform(String paramString, @SuppressWarnings("rawtypes") Map paramMap) {
		
		CallGraph cg = Scene.v().getCallGraph();
		
		//*********** Instance Type Analysis  ***********
		callInstanceTypeAnalysis();
//		System.out.println(printClasses());

		//*********** Views Analysis  ***********
		callViewsAnalysis();

		//*********** Processes Analysis  ***********
		Set<Process> processes = ProcessGenerator.generateProcesses(cg);
	
		//*********** Prepare information  ***********
		for(Process p : processes){
			ViewsCollector.putProcess(p);
			ArthoViewsCollector.putProcess(p);
		}
//		System.out.println(ViewsCollector.printMethodsViews());
		
		//*********** Dependency Analysis  ***********
		callDependencyAnalysis();
		
		//*********** Create all Sensors ***********
		ArthoSensor sensor = new ArthoSensor();
		sensor.pushViewInformation();
		
		Sensor[] our_sensors = new Sensor[SENSORS_NUMBER];
		our_sensors[0] = new ViewConsistencySensor();
		our_sensors[1] = new DepSensor();
		
		for(int i = 0; i < SENSORS_NUMBER; i++){
			our_sensors[i].pushViewInformation();
			our_sensors[i].pushDepInformation();
		}
//		System.out.println(((ViewConsistencySensor)our_sensors[0]).printMaximalViews());
		
		System.out.println("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ OUR APPROACH @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
		Set<Conflict> conflicts = new HashSet<Conflict>();
		
		//For each sensor
		for(int i = 0; i < SENSORS_NUMBER; i++){
			//For each conflict
			for(Conflict c : our_sensors[i].getConflicts()){
				conflicts.add(c);
			}
		}
		Set<IConflictGroup> groupedConflicts = groupConflicts(conflicts);
		
		System.out.println(printConflicts(groupedConflicts));
		
		System.out.println("\n\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ ARTHO's APPROACH @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");

		groupedConflicts = groupConflicts(sensor.getConflicts());
		System.out.println(printConflicts(groupedConflicts));
		
		System.out.println("\n\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ NATIVE METHODS @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
		System.out.println(printNativeMethods());
		System.out.println(printMethodsWithAnnotations());
		System.out.println(printExcludedMethods());
	}
	
	private String printClasses(){
		String result = "\n\n-------------- Classes Analysis ----------------\n";
		for (Entry<Unit, Set<String>> e : UnitsCollector.getResults()) {
			result += "\nUnit = "+ e.getKey()+"\nClasses:\n";
			for(String s : e.getValue()){
				result += s +"\n";
			}
		}
		return result;
	}
	private String printDependencies(){
		String result = "\n\n-------------- Dependency Analysis ----------------\n\n";
		result += VarDepCollector.getGraph().toString()+"\n--\n";
		result += ViewDepCollector.printDeps();
		return result;
	}
	private String printConflicts(Set<IConflictGroup> groupedConflicts) {
		String result = "";
		
		int size = groupedConflicts.size();
		if(size == 0)
			return result + "\tThere are no conflicts\n";
		else if(size == 1)
			result += "\tThere is 1 conflict:\n";
		else{
			result += "\tThere are "+size+" conflicts:\n";
		}
		for(IConflictGroup c : groupedConflicts){
			result += "\n"+c+"\n\n";
		}
		return result;
	}
	private String printNativeMethods() {
		String result = "\n-------------- Native Methods ----------------\n\n";
		if(NativesCollector.getNumberOfNatives() == 0){
			result += "\tThere are no native methods\n";
		}
		else{
			for(MethodName m : NativesCollector.getNativeMethods()){
				result += m.getObjClass()+"\t"+m.getMethodSignature()+"\n";
			}
		}
		result+= "\n----------------------------------------------\n\n";
		return result;
	}
	private String printMethodsWithAnnotations() {
		String result = "\n------------ Methods With Annotations ---------------\n\n";
		if(NativesCollector.getNumberOfUsedNatives() == 0){
			result += "\tThere are no native methods\n";
		}
		else{
			for(MethodName m : NativesCollector.getUsedMethods()){
				result += m.getObjClass()+"\t"+m.getMethodSignature()+"\n";
			}
		}
		result+= "\n----------------------------------------------\n\n";
		return result;
	}
	private String printExcludedMethods() {
		String result = "\n------------- Excluded Methods ---------------\n\n";
		if(NativesCollector.getNumberOfExcludedNatives() == 0){
			result += "\tThere are no native methods\n";
		}
		else{
			for(MethodName m : NativesCollector.getExcludedMethods()){
				result += m.getObjClass()+"\t"+m.getMethodSignature()+"\n";
			}
		}
		result+= "\n----------------------------------------------\n\n";
		return result;
	}
	
	/**
	 * This method groups all the conflicts to avoid multiple reporting
	 * @param conflicts set of conflicts that were detect
	 * @return set of groups of conflicts
	 */
	private Set<IConflictGroup> groupConflicts(Set<Conflict> conflicts) {
		Set<IConflictGroup> groupedConflicts = new HashSet<IConflictGroup>();
		for(Conflict c : conflicts){
			boolean added = false;
			for(IConflictGroup group : groupedConflicts){
				added = group.addConflict(c);
				if(added){
					break;
				}
			}
			if(!added){
				IConflictGroup newGroup = null;
				if(c instanceof ViewConflict){
					 newGroup = new ViewGroup((ViewConflict)c);
				}
				else if(c instanceof DepConflict){
					 newGroup = new DepGroup((DepConflict)c);
				}
				else{
					System.err.println("AnalysisTransformer: ERROR! Conflict is " +
							"neighter a ViewConflict nor a DepConflict:\n"
							+c+"("+c.getClass()+")");
					System.exit(-1);
				}
				groupedConflicts.add(newGroup);
			}
		}
		return groupedConflicts;
	}
}
